import os


def clone_repository(url, target):
    os.system("git clone {} {} > /dev/null 2>&1".format(url, target))


def clone_repositories(config_data):
    """
    clones the repositories as specified in the config data
    """

    workspace_name = config_data["name"]
    workspace_path = config_data["workspace_path"]

    repositories = config_data["repositories"]

    for repository in repositories:
        clone_name = repository["clone_name"]
        clone_path = repository["clone_path"]
        repo_url = repository["repo_url"]

        clone_location = os.path.join(
            workspace_path, workspace_name, clone_path, clone_name
        )

        if not os.path.exists(clone_location):
            print("Cloning repository {} to path {}".format(
                repo_url, clone_location
            ))

            clone_repository(repo_url, clone_location)
            continue

        print("Skipping {} as path already present".format(
            repo_url
        ))



import os
import json
import shutil
from .sources import AwsSource, LocalSource
from .repositories import clone_repositories


def setup_from_template(workspace_name, workspace_path, config_folder):
    """
    creates the workspace in specific path based on the template
    """

    src_path = "structure"
    dst_path = os.path.join(workspace_path, workspace_name)

    if os.path.exists(dst_path):
        print("Skipping creation as workspace {} in path {} already exists".format(
            workspace_name, workspace_path)
        )
        return

    print("Creating workspace {}".format(workspace_name))
    shutil.copytree(src_path, dst_path)

    with open("config_template.json") as f:
        data = json.load(f)
    
    data["name"] = workspace_name
    data["workspace_path"] = workspace_path

    config_path = "{}/{}_config.json".format(config_folder, workspace_name)

    with open(config_path, "w") as f:
        json.dump(data, f)


def backup_data(config_path):
    """
    calls the appropriate backup data class depending
    on enabled sources
    """

    with open(config_path, 'r') as f:
        config_data = json.load(f)

    class_mapping = {
        "s3": AwsSource(config_data),
        "local": LocalSource(config_data)
    }

    enabled_sources = config_data["source_settings"]["enabled_sources"]

    for src in enabled_sources:
        print("Backing up data using {}".format(src))
        class_mapping[src].backup_data()


def restore_data(config_path):
    """
    calls the appropriate restore data class depending
    on enabled sources
    """

    with open(config_path, 'r') as f:
        config_data = json.load(f)

    class_mapping = {
        "s3": AwsSource(config_data),
        "local": LocalSource(config_data)
    }

    enabled_sources = config_data["source_settings"]["enabled_sources"]

    clone_repositories(config_data)

    for src in enabled_sources:
        print("Backing up data using {}".format(src))
        class_mapping[src].restore_data()



import os
import argparse
from .runners import backup_data, restore_data, setup_from_template
from .load_config import is_config_loadable


def restore_workspace(args):
    is_config_loadable(args.config_path)
    restore_data(args.config_path)


def backup_workspace(args):
    is_config_loadable(args.config_path)
    backup_data(args.config_path)


def create_new_workspace(args):
    setup_from_template(args.name, args.workspace_path, args.config_folder)


def cli():
    """
    defines the entire cli parsing components of wbck
    """

    parser = argparse.ArgumentParser(
        description='Application to backup and restore my workspace data using config files'
    )
    subparsers = parser.add_subparsers(title="Commands")

    create_parser = subparsers.add_parser(
        "create",
        help="creates a new workspace"
    )
    create_parser.add_argument(
        "--name",
        required=True,
        help="name of the new workspace"
    )
    create_parser.add_argument(
        "--workspace-path",
        default=os.path.expanduser("~"),
        help="path where the workspace needs to be created"
    )
    create_parser.add_argument(
        "--config-folder",
        required=True,
        help="path where the config for this workspace is"
    )
    create_parser.set_defaults(func=create_new_workspace)

    backup_parser = subparsers.add_parser(
        "backup",
        help="commands pertaining to backup of your workspaces"
    )
    backup_parser.add_argument(
        "--config-path",
        required=True,
        help="path where the config for this workspace is"
    )
    backup_parser.set_defaults(func=backup_workspace)

    restore_parser = subparsers.add_parser(
        "restore",
        help="commands pertaining to restore of your workspaces"
    )
    restore_parser.add_argument(
        "--config-path",
        required=True,
        help="path where the config for this workspace is"
    )
    restore_parser.set_defaults(func=restore_workspace)

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    cli()



import os


def zipdir(path, ziph, ignore_files=[]):
    for root, dirs, files in os.walk(path):
        for file in files:
            if file in ignore_files:
                continue
            ziph.write(
                os.path.join(root, file),
                os.path.relpath(
                    os.path.join(root, file),
                    os.path.join(path, '..')
                )
            )



import boto3

from .base import BaseSource


class AwsSource(BaseSource):

    def __init__(self, config_data):
        super().__init__(config_data)
        self.AWS_KEY = self.source_settings["s3"]["aws_key"]
        self.AWS_SECRET = self.source_settings["s3"]["aws_secret"]

        self.BUCKET_NAME = self.source_settings["s3"]["bucket_name"]

    def backup_data(self):
        """
        backs up data to the AWS s3 bucket as per configuration
        """

        self.generate_compressed_data()

        print("======================> Uploading file {} to bucket {}".format(
            self.zip_name, self.BUCKET_NAME))

        s3 = boto3.client(
            's3',
            aws_access_key_id=self.AWS_KEY,
            aws_secret_access_key=self.AWS_SECRET
        )

        s3.upload_file(self.zip_name, self.BUCKET_NAME, self.zip_name)

        self.perform_cleanup()


    def restore_data(self):
        """
        restores data from AWS s3 bucket as per configuration
        """

        print("======================> Downloading file {} from bucket {}".format(
            self.zip_name, self.BUCKET_NAME))

        s3 = boto3.client(
            's3',
            aws_access_key_id=self.AWS_KEY,
            aws_secret_access_key=self.AWS_SECRET
        )

        s3.download_file(self.BUCKET_NAME, self.zip_name, self.zip_name)
        
        self.extract_from_compressed_data()

        self.perform_cleanup()



import os
import shutil
from .base import BaseSource


class LocalSource(BaseSource):

    def __init__(self, config_data):
        super().__init__(config_data)
        
        self.local_path = self.source_settings["local"]["local_path"]

    def backup_data(self):
        """
        backs up data to a local path specified in the configurations
        """

        if not os.path.exists(self.local_path):
            raise FileNotFoundError(f"{self.local_path} does not exist")

        self.generate_compressed_data()

        print("======================> Uploading file {} to folder {}".format(
            self.zip_name, self.local_path))

        shutil.copy(self.zip_name, self.local_path)

        self.perform_cleanup()

    def restore_data(self):
        """
        restores data from a local path specified in the configurations
        """

        print("======================> Downloading file {} from folder {}".format(
            self.zip_name, self.local_path))

        backup_path = os.path.join(self.local_path, self.zip_name)

        shutil.copy(backup_path, self.zip_name)

        self.extract_from_compressed_data()

        self.perform_cleanup()



import os
import shutil
import zipfile
from ..utils import zipdir
from datetime import datetime


class BaseSource(object):
    """
    This is the base source class serving as the base
    constract for all the other sources where from the backup
    or restoration needs to happen
    """

    def __init__(self, config_data):
        self.config_data = config_data
        self.source_settings = config_data["source_settings"]

        self.workspace_name = self.config_data["name"]
        self.workspace_path = os.getenv("HOME") if not config_data["workspace_path"] else \
            config_data["workspace_path"]
        
        self.zip_name = "{}-{}.zip".format(
            self.workspace_name,
            datetime.now().date().isoformat()
        )
        self.tmp_path = os.path.join(self.workspace_path, self.workspace_name, 'tmp')

    def backup_data(self):
        raise NotImplementedError()
    
    def restore_data(self):
        raise NotImplementedError()
    
    def perform_cleanup(self):
        """
        performs cleanup of the artifacts generated during the backup process
        """

        os.remove(self.zip_name)
        shutil.rmtree(self.tmp_path)
    
    def generate_compressed_data(self):
        """
        generates the compressed version of the data to backup
        """

        if os.path.exists(self.tmp_path):
            shutil.rmtree(self.tmp_path)

        if not os.path.exists(self.tmp_path):
            os.system("mkdir -p {}".format(self.tmp_path))

        for folder in self.source_settings["folders_to_maintain"]:
            src_path = os.path.join(self.workspace_path, self.workspace_name, folder)
            dst_path = os.path.join(self.tmp_path, folder)

            if os.path.exists(dst_path):
                continue

            print(f"Copying {src_path} to tmp location")
            shutil.copytree(src_path, dst_path)

        print(f"Generating zip file {self.zip_name}")
        zipf = zipfile.ZipFile(
            self.zip_name, 'w', zipfile.ZIP_DEFLATED
        )
        zipdir(self.tmp_path, zipf, self.source_settings['files_to_exclude'])
        zipf.close()

    def extract_from_compressed_data(self):
        """
        extracts the compressed version of the data to specific paths
        as per configuration
        """

        with zipfile.ZipFile(self.zip_name, 'r') as zip_ref:
            zip_ref.extractall(os.path.join(self.workspace_path, self.workspace_name))

        for folder in self.source_settings["folders_to_maintain"]:
            tmp_path = os.path.join(self.workspace_path, self.workspace_name, 'tmp')
            dst_path = os.path.join(self.workspace_path, self.workspace_name, folder)
            src_path = os.path.join(tmp_path, folder)

            if not os.path.exists(src_path):
                os.system("mkdir -p {}".format(src_path))

            if os.path.exists(dst_path):
                continue

            shutil.copytree(src_path, dst_path)


import json


def _is_enabled(_file):
    data = json.load(open(_file))

    return bool(data["enabled"])


def is_config_loadable(config_path):
    if not _is_enabled(config_path):
        raise SystemError("The configuration file is not enabled")



import os
from setuptools import setup, find_packages

with open("README.md", "r", encoding = "utf-8") as fh:
    long_description = fh.read()


def get_version():
    if not os.path.exists("VERSION"):
        raise FileNotFoundError("Version file does not exist")
    
    with open("VERSION", "r") as f:
        version = f.read()

    return version


setup(
    name='wbck',
    version=get_version(),
    author="Arpit Goyal",
    author_email="arpitgoyal.iitkgp@outlook.com",
    description="a command line utility to perform workspace backup",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/argoyal/wbck.git",
    packages=find_packages(),
    install_requires=[
        'boto3'
    ],
    entry_points={
        'console_scripts': [
            'wbck = wbck:cli',
        ]
    },
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License"
    ]
)
